#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2020 Lucas Heitzmann Gabrielli.
# This file is part of gdstk, distributed under the terms of the
# Boost Software License - Version 1.0.  See the accompanying
# LICENSE file or <http://www.boost.org/LICENSE_1_0.txt>

import pytest
import numpy
import gdstk

from conftest import assert_same_shape, assert_close


def test_init():
    path = gdstk.RobustPath(0j, 2)
    assert path.layers == (0,)
    assert path.datatypes == (0,)
    assert path.num_paths == 1
    assert path.size == 0
    path.set_layers(1)
    path.set_datatypes(2)
    assert path.layers == (1,)
    assert path.datatypes == (2,)

    path = gdstk.RobustPath(0j, [2, 2], layer=3, datatype=[4, 5])
    assert path.layers == (3, 3)
    assert path.datatypes == (4, 5)
    assert path.num_paths == 2
    assert path.size == 0
    path.set_layers(5, 4)
    path.set_datatypes(3, 2)
    assert path.layers == (5, 4)
    assert path.datatypes == (3, 2)

    path = gdstk.RobustPath((1, 1), 2, [-1, 1]).horizontal(10)
    assert path.layers == (0, 0)
    assert path.datatypes == (0, 0)
    assert path.num_paths == 2
    assert path.size == 1

    path = gdstk.RobustPath(-1j, [2, 2], [-1, 1], layer=3, datatype=[4, 5])
    assert path.layers == (3, 3)
    assert path.datatypes == (4, 5)
    assert path.num_paths == 2


def test_transforms(proof_cells):
    for scale_width in [True, False]:
        path0 = gdstk.RobustPath(
            0j,
            [0.1, 0.2],
            0.3,
            tolerance=1e-4,
            scale_width=scale_width,
        )
        path0.vertical(1).horizontal(0.5).turn(0.4, -numpy.pi, [0.2, 0.1]).segment(
            (-0.2, 0), relative=True
        )
        paths = [
            path0,
            path0.copy().mirror((1.5, 0)),
            path0.copy().mirror((1.5, 0), (1.5, 1)),
            path0.copy().scale(2, (3, 0)),
            path0.copy().scale(-2, (-1, 0)),
            path0.copy().rotate(numpy.pi / 2, (2, 1)).translate(0.2, -0.3),
        ]
        assert_same_shape(
            proof_cells[f"RobustPath: scale_width {scale_width}"].polygons, paths
        )


@pytest.fixture(scope="session")
def robust_path():
    path = gdstk.RobustPath((-1, 0), [0.1, 0], 0, tolerance=1e-4)
    path.segment((1, 1), width=[0.2, 0.1], offset=0.2)
    path.arc(
        2, 0.5 * numpy.pi, 0, width=(0.15, "constant"), offset=[-0.1, (0, "smooth")]
    )
    return path


def test_position(robust_path):
    assert_close(robust_path.position(-0.1), (-1, 0))
    assert_close(robust_path.position(0), (-1, 0))
    assert_close(robust_path.position(0.5), (0, 0.5))
    assert_close(robust_path.position(1), (1, 1))
    assert_close(robust_path.position(1.5), (1 + 2 ** 0.5, -1 + 2 ** 0.5))
    assert_close(robust_path.position(2), (3, -1))
    assert_close(robust_path.position(2.1), (3, -1))


def test_gradient(robust_path):
    assert_close(robust_path.gradient(-0.1), (2, 1))
    assert_close(robust_path.gradient(0), (2, 1))
    assert_close(robust_path.gradient(0.5), (2, 1))
    assert_close(robust_path.gradient(1), (2, 1))
    assert_close(robust_path.gradient(1, False), (numpy.pi, 0))
    assert_close(robust_path.gradient(1.5), (numpy.pi / 2 ** 0.5, -numpy.pi / 2 ** 0.5))
    assert_close(robust_path.gradient(2), (0, -numpy.pi))
    assert_close(robust_path.gradient(2, False), (0, -numpy.pi))
    assert_close(robust_path.gradient(2.1), (0, -numpy.pi))


def test_widths(robust_path):
    assert_close(robust_path.widths(-0.1), (0.1, 0))
    assert_close(robust_path.widths(0), (0.1, 0))
    assert_close(robust_path.widths(0.5), (0.15, 0.05))
    assert_close(robust_path.widths(1), (0.2, 0.1))
    assert_close(robust_path.widths(1, False), (0.15, 0.15))
    assert_close(robust_path.widths(1.5), (0.15, 0.15))
    assert_close(robust_path.widths(2), (0.15, 0.15))
    assert_close(robust_path.widths(2, False), (0.15, 0.15))
    assert_close(robust_path.widths(2.1), (0.15, 0.15))


def test_offsets(robust_path):
    assert_close(robust_path.offsets(-0.1), (0, 0))
    assert_close(robust_path.offsets(0), (0, 0))
    assert_close(robust_path.offsets(0.5), (-0.05, 0.05))
    assert_close(robust_path.offsets(1), (-0.1, 0.1))
    assert_close(robust_path.offsets(1, False), (-0.1, 0.1))
    assert_close(robust_path.offsets(1.5), (-0.1, 0.05))
    assert_close(robust_path.offsets(2), (-0.1, 0))
    assert_close(robust_path.offsets(2, False), (-0.1, 0))
    assert_close(robust_path.offsets(2.1), (-0.1, 0))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2020 Lucas Heitzmann Gabrielli.
# This file is part of gdstk, distributed under the terms of the
# Boost Software License - Version 1.0.  See the accompanying
# LICENSE file or <http://www.boost.org/LICENSE_1_0.txt>

import pytest
import numpy
import gdstk

from conftest import assert_same_shape, assert_close


def test_noreference():
    name = "ca_noreference"
    ref = gdstk.Reference(name, (1, -1), numpy.pi / 2, 2.1, True)
    assert ref.cell == name
    assert ref.bounding_box() is None
    assert ref.origin == (1, -1)
    assert ref.rotation == numpy.pi / 2
    assert ref.magnification == 2.1
    assert ref.x_reflection == True


def test_empty():
    name = "ca_empty"
    c = gdstk.Cell(name)
    ref = gdstk.Reference(c, (1, -1), numpy.pi / 2, 2, True, 2, 3, (3, 2))
    assert ref.cell is c
    assert ref.bounding_box() is None


def test_notempty():
    name = "ca_notempty"
    c = gdstk.Cell(name)
    ref = gdstk.Reference(c, (1, -1), numpy.pi / 2, 2, True, 2, 3, (3, 2))
    ref.origin = (0, 0)
    c.add(gdstk.rectangle((0, 0), (1, 2), 2, 3))
    assert_close(ref.bounding_box(), ((0, 0), (8, 5)))
    assert_same_shape(
        [gdstk.rectangle((0, 0), (8, 2)), gdstk.rectangle((0, 3), (8, 5))],
        gdstk.Cell("TMP").add(ref).flatten().polygons,
    )


def test_label_bounding_box():
    c = gdstk.Cell("CELL")
    l = gdstk.Label("Label", (2, 3))
    c.add(l)
    bb = c.bounding_box();
    assert bb[0][0] == 2 and bb[0][1] == 3
    assert bb[1][0] == 2 and bb[1][1] == 3
    ref = gdstk.Reference(c, (-1, 1))
    bb = ref.bounding_box()
    assert bb[0][0] == 1 and bb[0][1] == 4
    assert bb[1][0] == 1 and bb[1][1] == 4
    ang = numpy.pi / 4
    x = ref.origin[0] + l.origin[0] * numpy.cos(ang) - l.origin[1] * numpy.sin(ang)
    y = ref.origin[1] + l.origin[0] * numpy.sin(ang) + l.origin[1] * numpy.cos(ang)
    ref.rotation = ang
    bb = ref.bounding_box()
    assert_close(bb, ((x, y), (x, y)))

